Package: SteadyStateBVAR
Title: Bayesian Vector Autoregressions with Steady-State Priors
Version: 0.1.0
Authors@R: 
    person("Mark", "Becker", email = "mark.jw.becker@gmail.com", role = c("aut", "cre", "cph"))
Description: Provides estimation of Bayesian vector autoregression
    (BVAR) models with steady-state priors via 'Stan', along with
    functions for unconditional and conditional forecasting, as well as impulse
    response analysis. For details on the steady-state BVAR model
    see Villani (2009) <doi:10.1002/jae.1065>.
License: GPL (>= 3)
URL: https://github.com/markjwbecker/SteadyStateBVAR,
        https://markjwbecker.github.io/SteadyStateBVAR/
Depends: R (>= 3.6.0)
Imports: graphics, grDevices, methods, MTS (>= 1.2.1), Rcpp (>=
        0.12.0), RcppParallel (>= 5.0.1), rstan (>= 2.32.7), rstantools
        (>= 2.6.0), stats, utils
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
RoxygenNote: 8.0.0
Biarch: true
LinkingTo: BH (>= 1.66.0), Rcpp (>= 0.12.0), RcppEigen (>= 0.3.3.3.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.18.1), StanHeaders (>=
        2.18.0)
SystemRequirements: GNU make
BugReports: https://github.com/markjwbecker/SteadyStateBVAR/issues
NeedsCompilation: yes
Packaged: 2026-07-17 10:10:31 UTC; markj
Author: Mark Becker [aut, cre, cph]
Maintainer: Mark Becker <mark.jw.becker@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-24 10:30:10 UTC
