# Generated by roxygen2: do not edit by hand

S3method(summary,bvar)
export(IRF)
export(bvar)
export(conditional_forecast)
export(fit)
export(forecast)
export(ppi)
export(priors)
export(restrict_beta)
export(setup)
export(stochastic_volatility_plot)
import(Rcpp)
import(methods)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,title)
importFrom(rstan,sampling)
importFrom(rstantools,rstan_config)
importFrom(stats,embed)
importFrom(stats,frequency)
importFrom(stats,is.ts)
importFrom(stats,median)
importFrom(stats,plot.ts)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,setNames)
importFrom(stats,start)
importFrom(stats,time)
importFrom(stats,ts)
importFrom(stats,ts.plot)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(SteadyStateBVAR, .registration = TRUE)
