# Precompiled vignettes (too long runtime)

library(knitr)
# knit("vignettes/Homoscedastic-steady-state-BVAR.Rmd.orig", "vignettes/Homoscedastic-steady-state-BVAR.Rmd")
# fs::dir_copy("figure", "vignettes/figure", overwrite = TRUE)
# fs::dir_delete("figure")
knit("vignettes/RW-stochastic-volatility-steady-state-BVAR.Rmd.orig", "vignettes/RW-stochastic-volatility-steady-state-BVAR.Rmd")
fs::dir_copy("figure", "vignettes/figure", overwrite = TRUE)
fs::dir_delete("figure")
knit("vignettes/AR1-stochastic-volatility-steady-state-BVAR.Rmd.orig", "vignettes/AR1-stochastic-volatility-steady-state-BVAR.Rmd")
fs::dir_copy("figure", "vignettes/figure", overwrite = TRUE)
fs::dir_delete("figure")
