#' @keywords internal
"_PACKAGE"
## usethis namespace: start
#' @importFrom grDevices rgb
#' @importFrom graphics abline axis lines par points polygon title legend
#' @importFrom stats embed frequency plot.ts qnorm quantile rnorm setNames start time ts ts.plot is.ts median
#' @importFrom utils head tail
#' @import Rcpp
#' @import methods
#' @importFrom rstan sampling
#' @importFrom rstantools rstan_config
#' @importFrom RcppParallel RcppParallelLibs
#' @useDynLib SteadyStateBVAR, .registration = TRUE
## usethis namespace: end
NULL