% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{Villani2009}
\alias{Villani2009}
\title{Villani (2009) dataset}
\format{
\subsection{\code{Villani2009}}{

A multivariate time series object with 104 rows and 7 columns:
\describe{
\item{delta y_f}{foreign GDP growth}
\item{pi_f}{foreign CPI inflation}
\item{i_f}{foreign 3-month interest rate}
\item{delta y}{domestic GDP growth}
\item{pi}{domestic CPI inflation}
\item{i}{domestic 3-month interest rate}
\item{q}{level of the real exchange rate defined as \eqn{q = s + p_f - p}, where \eqn{p_f} and \eqn{p}
are the foreign and domestic CPI levels in logs and \eqn{s} is the log of the trade-weighted nominal exchange rate}
}
}
}
\source{
Villani, M. (2009). Steady-state priors for vector autoregressions.
\emph{Journal of Applied Econometrics}, 24(4), pp. 630-650.
}
\usage{
Villani2009
}
\description{
Swedish macroeconomic data (1980Q1–2005Q4) from Section 4.1 in Villani (2009)
}
\keyword{datasets}
