% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SteadyStateBVAR-package.R
\docType{package}
\name{SteadyStateBVAR-package}
\alias{SteadyStateBVAR}
\alias{SteadyStateBVAR-package}
\title{SteadyStateBVAR: Bayesian Vector Autoregressions with Steady-State Priors}
\description{
Provides estimation of Bayesian vector autoregression (BVAR) models with steady-state priors via 'Stan', along with functions for unconditional and conditional forecasting, as well as impulse response analysis. For details on the steady-state BVAR model see Villani (2009) \doi{10.1002/jae.1065}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/markjwbecker/SteadyStateBVAR}
  \item \url{https://markjwbecker.github.io/SteadyStateBVAR/}
  \item Report bugs at \url{https://github.com/markjwbecker/SteadyStateBVAR/issues}
}

}
\author{
\strong{Maintainer}: Mark Becker \email{mark.jw.becker@gmail.com} [copyright holder]

Authors:
\itemize{
  \item Mark Becker \email{mark.jw.becker@gmail.com} [copyright holder]
}

}
\keyword{internal}
