% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/setup.R
\name{setup}
\alias{setup}
\title{Set up the steady-state BVAR model}
\usage{
setup(
  x,
  p = 1,
  deterministic = c("constant", "constant_and_dummy", "constant_and_trend"),
  dummy = NULL
)
}
\arguments{
\item{x}{A steady-state \code{bvar} object created by \code{\link{bvar}}.}

\item{p}{Integer. The lag order of the VAR. Default \code{1}.}

\item{deterministic}{Character. The deterministic component to include.
One of \code{"constant"} (default), \code{"constant_and_dummy"}, or
\code{"constant_and_trend"}.}

\item{dummy}{Numeric vector of a dummy variable. Only
used when \code{deterministic = "constant_and_dummy"}. Default \code{NULL}.}
}
\value{
The steady-state \code{bvar} object with a \code{setup} list containing the
required components for prior specification and estimation, and also the OLS estimates.
}
\description{
Prepares the components needed for prior specification and estimation.
Also computes OLS estimates.
}
\examples{
yt <- matrix(rnorm(50), 25, 2)

bvar_obj <- bvar(data = yt)

bvar_obj <- setup(bvar_obj)
}
