% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{KoopKorobilis2010}
\alias{KoopKorobilis2010}
\title{Koop and Korobilis (2010) dataset}
\format{
\subsection{\code{KoopKorobilis2010}}{

A multivariate time series object with 215 rows and 3 columns:
\describe{
\item{delta pi}{inflation rate (annual percentage change in a chain-weighted GDP price index)}
\item{u}{unemployment rate (seasonally adjusted civilian unemployment rate, all civilian workers aged 16 years or older)}
\item{r}{interest rate (yield on the three-month Treasury bill rate)}
}
}
}
\source{
Koop, G. and Korobilis, D. (2010). Bayesian multivariate time series
methods for empirical macroeconomics. \emph{Foundations and Trends in Econometrics}, 3(4), pp. 267-358.
}
\usage{
KoopKorobilis2010
}
\description{
Quarterly US macroeconomic data (1953Q1-2006Q3) from Koop and Korobilis (2010)
}
\keyword{datasets}
